// Package implement ...
//
// Description : implement ...
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 2025-05-06 11:53
package implement

import (
	"git.zhangdeman.cn/zhangdeman/consts"
	"git.zhangdeman.cn/zhangdeman/json_filter/abstract"
	"git.zhangdeman.cn/zhangdeman/serialize"
	"github.com/tidwall/sjson"
	"reflect"
	"sync"
)

func NewSjsonWrite() abstract.IJsonWrite {
	return &SjsonWrite{
		res: "",
		l:   &sync.RWMutex{},
	}
}

type SjsonWrite struct {
	res string
	l   *sync.RWMutex
}

func (s *SjsonWrite) Set(dataPath string, data any) error {
	var (
		err error
	)

	s.l.Lock()
	defer s.l.Unlock()
	existResRead := NewGjsonRead(s.res)
	if !existResRead.Exist(dataPath) || data == nil {
		// 路径不存在
		if s.res, err = sjson.Set(s.res, dataPath, data); nil != err {
			return err
		}
		return nil
	}
	// 路径已存在, 判断是否为map
	if existResRead.Type(dataPath) != consts.DataTypeMapStrAny.String() {
		if s.res, err = sjson.Set(s.res, dataPath, data); nil != err {
			return err
		}
		return nil
	}
	// 判断data是否为map
	dataType := reflect.TypeOf(data)
	if dataType.Kind() == reflect.Ptr {
		dataType = dataType.Elem()
	}
	if dataType.Kind() != reflect.Struct && dataType.Kind() != reflect.Map {
		// 既不是map, 也不是struct
		if s.res, err = sjson.Set(s.res, dataPath, data); nil != err {
			return err
		}
		return nil
	}
	// 路径已存在, 已存在数据为map, 且要设置的数据也为map, 进行数据合并
	mergeDataMap := map[string]any{}
	existMapVal := map[string]any{}

	if existMapVal, err = existResRead.Map(dataPath); nil != err {
		return err
	}
	// 合并输入数据
	if err = serialize.JSON.MergeDataForReceiver(&mergeDataMap, existMapVal, data); nil != err {
		return err
	}
	if s.res, err = sjson.Set(s.res, dataPath, mergeDataMap); nil != err {
		return err
	}
	return nil
}

func (s *SjsonWrite) Result() string {
	return s.res
}

func (s *SjsonWrite) Map() (map[string]any, error) {
	var (
		mapRes map[string]any
		err    error
	)
	if err = s.MapWithReceiver(&mapRes); nil != err {
		return nil, err
	}
	return mapRes, nil
}

func (s *SjsonWrite) MapWithReceiver(receiver any) error {
	return serialize.JSON.UnmarshalWithNumberForString(s.res, receiver)
}

func (s *SjsonWrite) Array() ([]any, error) {
	var (
		arrRes []any
		err    error
	)
	if err = s.ArrayWithReceiver(&arrRes); nil != err {
		return nil, err
	}
	return arrRes, nil
}

func (s *SjsonWrite) ArrayWithReceiver(receiver any) error {
	return serialize.JSON.UnmarshalWithNumberForString(s.res, receiver)
}
